<?php

namespace Drupal\acsf_backup_manager\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Routing\TrustedRedirectResponse;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Class ActionController.
 *
 * Provide actions for the routes.
 */
class ActionController extends ControllerBase {

  /**
   * The API client.
   *
   * @var \Drupal\acsf_backup_manager\Controller\ApiInterface
   */
  protected $client;

  /**
   * File system service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * {@inheritdoc}
   */
  protected $config;

  /**
   * {@inheritdoc}
   */
  protected $limit;

  /**
   * {@inheritdoc}
   */
  protected $debug;

  /**
   * MediaFileController constructor.
   *
   * @param \Drupal\acsf_backup_manager\Controller\ApiInterface $client
   *   The API client.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   *   Request stack variable.
   */
  public function __construct(ApiInterface $client, ConfigFactoryInterface $config_factory, RequestStack $requestStack) {
    $this->client = $client;
    $this->configFactory = $config_factory;
    $this->config = $this->configFactory->get('acsf_backup_manager.settings');
    $this->debug = $this->config->get('debug');
    $this->limit = $this->config->get('limit');
    $this->request = $requestStack->getCurrentRequest();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('acsf_backup_manager.api_client'),
      $container->get('config.factory'),
      $container->get('request_stack')
    );
  }

  /**
   * Invokes creating backup process.
   *
   * @param int $siteId
   *   Site ID from ACSF.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   Redirect returned.
   */
  public function backupCreate(int $siteId) {
    $this->client->backupCreate($siteId);

    return $this->redirect('acsf_backup_manager.admin_backups_form', ['siteId' => $siteId]);
  }

  /**
   * Invokes deleting backup process.
   *
   * @param int $siteId
   *   Site ID from ACSF.
   * @param int $backupId
   *   Site ID from ACSF.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   Redirect returned.
   */
  public function backupDelete(int $siteId, int $backupId) {
    $this->client->backupDelete($siteId, $backupId);

    $referer = $this->request->server->get('HTTP_REFERER');
    return new TrustedRedirectResponse($referer, '301');
  }

  /**
   * Invokes downloading backup process.
   *
   * @param int $siteId
   *   Site ID from ACSF.
   * @param int $backupId
   *   Site ID from ACSF.
   *
   * @return \Drupal\Core\Routing\TrustedRedirectResponse
   *   Redirect returned.
   */
  public function backupDownload(int $siteId, int $backupId) {
    $tmpUrl = $this->client->backupGetTempLink($siteId, $backupId);

    return new TrustedRedirectResponse($tmpUrl, '301');
  }

  /**
   * Invokes deleting all site backup process.
   *
   * @param int $siteId
   *   Site ID from ACSF.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   Redirect returned.
   */
  public function backupsDelete(int $siteId) {
    $this->client->backupsDelete($siteId);

    $referer = $this->request->server->get('HTTP_REFERER');
    return new TrustedRedirectResponse($referer, '301');
  }

}
