<?php

namespace Drupal\acsf_backup_manager\Controller;

use Drupal\Core\Controller\ControllerBase;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\RequestException;
use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class ApiController.
 *
 * Provide with the possibility to work with some ACSF API.
 */
class ApiController extends ControllerBase implements ApiInterface {

  use HelperTrait;

  /**
   * Http client service.
   *
   * @var \GuzzleHttp\Client
   */
  protected $httpClient;

  /**
   * File system service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * {@inheritdoc}
   */
  protected $config;

  /**
   * {@inheritdoc}
   */
  protected $limit;

  /**
   * {@inheritdoc}
   */
  protected $debug;

  /**
   * MediaFileController constructor.
   *
   * @param \GuzzleHttp\Client $http_client
   *   Http client service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   */
  public function __construct(Client $http_client, ConfigFactoryInterface $config_factory) {
    $this->httpClient = $http_client;
    $this->configFactory = $config_factory;
    $this->config = $this->configFactory->get('acsf_backup_manager.settings');
    $this->debug = $this->config->get('debug');
    $this->limit = $this->config->get('limit');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('http_client'),
      $container->get('config.factory')
    );
  }

  /**
   * Helper.
   *
   * @param int $id
   *   Must be a number.
   *
   * @return bool
   *   True if given ID is valid (numeric and > 0).
   */
  public static function idCheck(int $id) {
    return is_numeric($id) && $id > 0;
  }

  /**
   * Helper.
   *
   * @return array
   *   Auth array API user and key.
   */
  protected function getRequestAuth() {
    return [
      'auth' => [
        $this->config->get('api_user'),
        $this->config->get('api_key'),
      ],
    ];
  }

  /**
   * Sends a request using the guzzle HTTP library; prints out any errors.
   *
   * @param string $url
   *   Request Url.
   * @param string $method
   *   Request method like GET, POST, DELETE, etc.
   * @param array $form_params
   *   Parameters of request.
   *
   * @return \Psr\Http\Message\ResponseInterface|null
   *   Returns response or null.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function request(string $url, string $method, array $form_params = []) {
    // We are setting http_errors => FALSE so that we can handle them ourselves.
    // Otherwise, we cannot differentiate between different HTTP status codes
    // since all 40X codes will just throw a ClientError exception.
    // $client = new Client(['http_errors' => FALSE]);.
    $parameters = $this->getRequestAuth();
    if ($form_params) {
      $parameters['form_params'] = $form_params;
    }

    try {
      $res = $this->httpClient->request($method, $url, $parameters);
      return $res;
    }
    catch (RequestException $e) {
      if ($this->debug) {
        \Drupal::logger('acsf_backup_manager')
          ->error("Request exception!\nError message:\n" . $e->getMessage());
        \Drupal::messenger()
          ->addMessage("Request exception!\nError message:\n" . $e->getMessage(), 'error');
      }
    }

    return NULL;
  }

  /**
   * Checks that server responds and authentication is successful.
   *
   * @return bool
   *   is a connection established.
   */
  public function checkConnection() {
    $url = $this->config->get('api_url') . 'ping';
    $res = $this->request($url, 'GET');

    if (!$res || $res->getStatusCode() != 200) {
      if ($this->debug) {
        \Drupal::logger('acsf_backup_manager')
          ->error("Check connection: Response not exists");
      }

      return FALSE;
    }
    elseif ($res->getStatusCode() == 200) {
      return TRUE;
    }
  }

  /**
   * Creates backup.
   *
   * @param int $siteId
   *   Site ID from ACSF.
   *
   * @return bool
   *   Boolean result returned.
   */
  public function backupCreate(int $siteId) {
    $url = $this->config->get('api_url') . 'sites/' . $siteId . '/backup';

    $backupScope = [
      'components' => [],
    ];
    foreach ($this->config->get('backup_scope') as $key => $value) {
      if ($value !== 0) {
        $backupScope['components'][] = $key;
      }
    }

    if (count($backupScope) == 0) {
      if ($this->debug) {
        \Drupal::logger('acsf_backup_manager')
          ->warning("Backup is skipped, because of empty backup scope. You should select at least one item in the settings.");
      }

      \Drupal::messenger()
        ->addMessage("Backup is skipped, because of empty backup scope. You should select at least one item in the settings.");

      return FALSE;
    }

    $res = $this->request($url, 'POST', $backupScope);

    if (!$res || $res->getStatusCode() != 200) {
      if ($this->debug) {
        \Drupal::logger('acsf_backup_manager')
          ->error("backupCreate: Response Error \n");
      }

      \Drupal::messenger()
        ->addMessage("backupCreate: Response Error \n", 'error');
      return FALSE;
    }
    else {
      $body = json_decode($res->getBody()->getContents());

      if ($this->debug) {
        \Drupal::logger('acsf_backup_manager')
          ->info("Backup creating process started successfully. It will take some time. \nTask ID: " . ($body->task_id ? $body->task_id : '<empty>'));
      }

      \Drupal::messenger()
        ->addMessage("Backup creating process started successfully. It will take some time.\nTask ID: " . ($body->task_id ? $body->task_id : '<empty>'));
    }

    return TRUE;
  }

  /**
   * Fetches the site using the Site Factory REST API.
   *
   * @param int $siteId
   *   ACSF site ID.
   *
   * @return false|mixed
   *   Returned site class or false.
   */
  public function getSite(int $siteId) {
    $url = $this->config->get('api_url') . 'sites/' . $siteId;

    $res = $this->request($url, 'GET');
    if (!$res || $res->getStatusCode() != 200) {
      if ($this->debug) {
        \Drupal::logger('acsf_backup_manager')
          ->error("Error whilst fetching site $siteId!\n");

        \Drupal::messenger()
          ->addMessage("Error whilst fetching site $siteId!\n", 'error');
      }
      return FALSE;
    }
    else {
      return json_decode($res->getBody()->getContents());
    }

  }

  /**
   * Fetches the list of all sites using the Site Factory REST API.
   */
  public function getAllSites() {
    // Starting from page 1.
    $page = 1;

    $sites = [];

    // Iterate through the paginated list until we get all sites, or
    // an error occurs.
    do {
      $url = $this->config->get('api_url') . 'sites/' . "?limit=" . $this->limit . "&page=" . $page;
      $has_another_page = FALSE;

      $res = $this->request($url, 'GET');

      if (!$res || $res->getStatusCode() != 200) {
        if ($this->debug) {
          \Drupal::logger('acsf_backup_manager')
            ->error("Error whilst fetching site list!\n");
        }
        \Drupal::messenger()
          ->addMessage("Error whilst fetching site list!\n", 'error');

        continue;
      }

      $next_page_header = $res->getHeader('link');
      $response = json_decode($res->getBody()->getContents());

      // If the next page header is present and has a "next" link, we know we
      // have another page.
      if (!empty($next_page_header) && strpos($next_page_header[0], 'rel="next"') !== FALSE) {
        $has_another_page = TRUE;
        $page++;
      }

      foreach ($response->sites as $site) {
        $sites[] = $site;
      }
    } while ($has_another_page);

    return $sites;
  }

  /**
   * Gets site backups.
   *
   * @param int $siteId
   *   ACSF Site ID.
   *
   * @return bool
   *   Boolean result returned.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function getSiteBackups(int $siteId) {
    $url = $this->config->get('api_url') . 'sites/' . $siteId . '/backups';
    $res = $this->request($url, 'GET');

    if (!$res || $res->getStatusCode() != 200) {
      return [];
    }

    $backups = json_decode($res->getBody()->getContents())->backups;

    if (!$backups) {
      if ($this->debug) {
        \Drupal::logger('acsf_backup_manager')
          ->info("No records found for the site ID $siteId started successfully");
      }

      return [];
    }

    return $backups;
  }

  /**
   * Gets all backups.
   *
   * @return array
   *   Array of backups returned.
   */
  public function getAllBackups() {
    $sites = $this->getAllSites();
    $backups = [];

    foreach ($sites as $site) {
      $siteId = $site->id;
      $backupsTmp = $this->getSiteBackups($siteId);

      if ($backupsTmp) {
        foreach ($backupsTmp as $backup) {
          $backups[] = $backup;
        }
      }
    }

    return $backups;
  }

  /**
   * Gets all orphan backups.
   *
   * @return array
   *   Array of backups returned.
   */
  public function getAllBackupsOrphan(int $siteIdFrom, int $siteIdTo) {
    $backups = [];

    for ($siteId = $siteIdFrom; $siteId <= $siteIdTo; $siteId++) {
      if (!$this->getSite($siteId)) {
        $backupsTmp = $this->getSiteBackups($siteId);

        if ($backupsTmp) {
          foreach ($backupsTmp as $backup) {
            $backups[] = $backup;
          }
        }
      }
    }

    return $backups;
  }

  /**
   * Deletes backup.
   *
   * @param int $siteId
   *   ACSF Site ID.
   * @param int $backupId
   *   ACSF Backup ID.
   *
   * @return bool
   *   Boolean result returned.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function backupDelete(int $siteId, int $backupId) {
    $url = $this->config->get('api_url') . 'sites/' . $siteId . '/backups/' . $backupId;
    $res = $this->request($url, 'DELETE');

    if (!$res || $res->getStatusCode() != 200) {
      if ($this->debug) {
        \Drupal::logger('acsf_backup_manager')
          ->error("Error! Whilst deleting backup ID $backupId. Please check the above messages for the full error.\n");
      }
      \Drupal::messenger()
        ->addMessage("Error! Whilst deleting backup ID $backupId. Please check the above messages for the full error.\n", 'error');

      return FALSE;
    }

    $task = json_decode($res->getBody()->getContents())->task_id;
    if ($this->debug) {
      \Drupal::logger('acsf_backup_manager')
        ->info("Backup ID $backupId deleting process for site ID $siteId started successfully\nTask ID: " . ($task ? $task : '<empty>'));
    }

    \Drupal::messenger()
      ->addMessage("Backup ID $backupId deleting process for site ID $siteId started successfully\nTask ID: " . ($task ? $task : '<empty>'));

    return TRUE;
  }

  /**
   * Gets temp download link for the backup.
   *
   * @param int $siteId
   *   ACSF Site ID.
   * @param int $backupId
   *   ACSF Backup ID.
   *
   * @return bool|mixed
   *   Url or false result returned.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function backupGetTempLink(int $siteId, int $backupId) {
    $url = $this->config->get('api_url') . 'sites/' . $siteId . '/backups/' . $backupId . '/url';
    $res = $this->request($url, 'GET');

    if (!$res || $res->getStatusCode() != 200) {
      if ($this->debug) {
        \Drupal::logger('acsf_backup_manager')
          ->error("Error! Whilst getting backup ID $backupId temp url. Please check the above messages for the full error.\n");
      }
      \Drupal::messenger()
        ->addMessage("Error! Whilst getting backup ID $backupId temp url. Please check the above messages for the full error.\n", 'error');

      return FALSE;
    }

    $tmpUrl = json_decode($res->getBody()->getContents())->url;
    if ($this->debug) {
      \Drupal::logger('acsf_backup_manager')
        ->info("Backup ID $backupId getting link process for site ID $siteId started successfully\nUrl: " . ($tmpUrl ? $tmpUrl : '<empty>'));
    }

    return $tmpUrl;
  }

  /**
   * Gets file size the backup.
   *
   * @param string $url
   *   File url.
   *
   * @return bool|mixed
   *   Size or false result returned.
   */
  public function backupGetFileSize(string $url) {
    $headers = get_headers($url, TRUE);

    $return = FALSE;
    if (isset($headers['Content-Length'])) {

      $return = [
        'original' => $headers['Content-Length'],
        'formatted' => $this->getFormattedFileSize($headers['Content-Length']),
      ];
    }

    return $return;
  }

  /**
   * Deletes site backups.
   *
   * @param int $siteId
   *   ACSF Site ID.
   *
   * @return bool
   *   Boolean result returned.
   */
  public function backupsDelete(int $siteId) {
    $backupRetention = $this->config->get('backup_retention');
    $backups = $this->getSiteBackups($siteId);

    if ($backups) {
      foreach ($backups as $backup) {
        $backupId = $backup->id;
        if ($this->isObsolete($backupRetention, $backup->timestamp)) {
          $this->backupDelete($siteId, $backupId);
        }
        else {
          // @todo Move all notifications (exclude exceptions) to the ActionController?
          if ($this->debug) {
            \Drupal::logger('acsf_backup_manager')
              ->info("Keeping backup ID $backupId since it was created sooner than $backupRetention days ago.\n");
          }

          \Drupal::messenger()
            ->addMessage("Keeping backup ID $backupId since it was created sooner than $backupRetention days ago.\n");
        }
      }

      return TRUE;
    }

    return FALSE;
  }

  /**
   * Fetches the list of all site groups using the Site Factory REST API.
   */
  public function getAllSiteGroups() {
    // Starting from page 1.
    $page = 1;

    $groups = [];

    // Iterate through the paginated list until we get all site groups, or
    // an error occurs.
    do {
      $url = $this->config->get('api_url') . 'groups/' . "?limit=" . $this->limit . "&page=" . $page;
      $has_another_page = FALSE;

      $res = $this->request($url, 'GET');

      if (!$res || $res->getStatusCode() != 200) {
        if ($this->debug) {
          \Drupal::logger('acsf_backup_manager')
            ->error("Error whilst fetching site groups list!\n");
        }
        \Drupal::messenger()
          ->addMessage("Error whilst fetching site groups list!\n", 'error');
        continue;
      }

      $next_page_header = $res->getHeader('link');
      $response = json_decode($res->getBody()->getContents());

      // If the next page header is present and has a "next" link, we know we
      // have another page.
      if (!empty($next_page_header) && strpos($next_page_header[0], 'rel="next"') !== FALSE) {
        $has_another_page = TRUE;
        $page++;
      }

      foreach ($response->groups as $group) {
        $groups[] = $group;
      }
    } while ($has_another_page);

    return $groups;
  }

  /**
   * Fetches the list of all site collections using the Site Factory REST API.
   */
  public function getAllSiteCollections() {
    // Starting from page 1.
    $page = 1;

    $collections = [];

    // Iterate through the paginated list until we get all user groups, or
    // an error occurs.
    do {
      $url = $this->config->get('api_url') . 'collections/' . "?limit=" . $this->limit . "&page=" . $page;
      $has_another_page = FALSE;

      $res = $this->request($url, 'GET');

      if (!$res || $res->getStatusCode() != 200) {
        if ($this->debug) {
          \Drupal::logger('acsf_backup_manager')
            ->error("Error whilst fetching site collections list!\n");
        }
        \Drupal::messenger()
          ->addMessage("Error whilst fetching site collections list!\n", 'error');

        continue;
      }

      $next_page_header = $res->getHeader('link');
      $response = json_decode($res->getBody()->getContents());

      // If the next page header is present and has a "next" link, we know we
      // have another page.
      if (!empty($next_page_header) && strpos($next_page_header[0], 'rel="next"') !== FALSE) {
        $has_another_page = TRUE;
        $page++;
      }

      foreach ($response->collections as $collection) {
        $collections[] = $collection;
      }
    } while ($has_another_page);

    return $collections;
  }

}
