<?php

namespace Drupal\acsf_backup_manager\Form;

use Drupal\acsf_backup_manager\Controller\ApiInterface;
use Drupal\acsf_backup_manager\Controller\HelperTrait;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Pager\PagerManagerInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\ResponseHeaderBag;

/**
 * Class AdminBackupsOrphanForm.
 *
 * Provides the form for the orphan backups.
 */
class AdminBackupsOrphanForm extends FormBase {
  use HelperTrait;

  /**
   * The API client.
   *
   * @var \Drupal\acsf_backup_manager\Controller\ApiInterface
   */
  protected $client;

  /**
   * The pager manager.
   *
   * @var \Drupal\Core\Pager\PagerManagerInterface
   */
  protected $pagerManager;

  /**
   * The request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * The items storage.
   *
   * @var array
   */
  protected $items;

  /**
   * File system service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * {@inheritdoc}
   */
  protected $config;

  /**
   * CallServicesController constructor.
   *
   * @param \Drupal\acsf_backup_manager\Controller\ApiInterface $client
   *   The API client.
   * @param \Drupal\Core\Pager\PagerManagerInterface $pagerManager
   *   The pager manager.
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   *   Request stack variable.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   */
  public function __construct(ApiInterface $client, PagerManagerInterface $pagerManager, RequestStack $requestStack, ConfigFactoryInterface $config_factory) {
    $this->client = $client;
    $this->pagerManager = $pagerManager;
    $this->request = $requestStack->getCurrentRequest();
    $this->configFactory = $config_factory;
    $this->config = $this->configFactory->get('acsf_backup_manager.settings');
  }

  /**
   * Services DI.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   Container DI.
   *
   * @return static
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('acsf_backup_manager.api_client'),
      $container->get('pager.manager'),
      $container->get('request_stack'),
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'acsf_backup_manager_admin_backups_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'acsf_backup_manager.backups',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $siteIdFrom = $this->request->get('siteIdFrom');
    $siteIdTo = $this->request->get('siteIdTo');

    $form['legend_orphan'] = [
      '#markup' => $this->t('Sometimes it happens that the backups are not tied to any specific site (since the site has been deleted). In this case, the search form for lost backups by site ID ranges will help you. You can determine the approximate search range based on the values of the current site IDs in your account.'),
    ];

    $form['filter'] = [
      '#name' => 'filter-details',
      '#type' => 'details',
      '#title' => $this->t('Filter'),
      '#open' => TRUE,
      '#attributes' => [
        'class' => [
          'acsf-backup-manager-filter',
        ],
      ],
    ];

    $form['filter']['site_id_from'] = [
      '#type' => 'number',
      '#title' => $this->t('From site ID'),
      '#size' => 15,
      '#default_value' => !empty($siteIdFrom) ? $siteIdFrom : 100,
    ];

    $form['filter']['site_id_to'] = [
      '#type' => 'number',
      '#title' => $this->t('To site ID'),
      '#size' => 15,
      '#default_value' => !empty($siteIdTo) ? $siteIdTo : 200,
    ];

    $form['filter']['submit'] = [
      '#name' => 'filter',
      '#type' => 'submit',
      '#value' => $this->t('Search'),
    ];

    $form['batch_action'] = [
      '#title' => $this->t("Action"),
      '#type' => 'select',
      '#options' => [
        '' => $this->t("-- Select action --"),
        'delete_backups_backups' => $this->t("Delete backup"),
        'export_backups_backups' => $this->t("Export backups report"),
      ],
    ];

    $backupRetention = $this->config->get('backup_retention');

    $form['legend'] = [
      '#markup' => '<span class="acsf-backup-manager-color-error">' . $this->t('Marked') . '</span> - ' . $this->t('are obsolete archives. You can define in the settings which archives are considered obsolete. Currently it is @backupRetention days.', ['@backupRetention' => $backupRetention]),
    ];

    $form['entities-table'] = [
      '#type' => 'table',
      '#empty' => $this->t('There are no items.'),
      // TableSelect: Injects a first column containing the selection widget
      // into each table row.
      // Note that you also need to set #tableselect on each form submit
      // button that relies on non-empty selection values (see below).
      '#tableselect' => TRUE,
    ];

    $form['entities-table']['#header'] = [
      $this->t('ID'),
      $this->t('Site ID'),
      $this->t('User ID'),
      $this->t('Created'),
      $this->t('Obsolete'),
      $this->t('Directory'),
      $this->t('File'),
    ];
    if ($this->config->get('filesize')) {
      $form['entities-table']['#header'][] = $this->t('Size');
    }
    $form['entities-table']['#header'][] = $this->t('Label');
    $form['entities-table']['#header'][] = $this->t('Component List');
    $form['entities-table']['#header'][] = $this->t('Operations');
    // Build the table rows and columns.
    // The first nested level in the render array forms the table row,
    // on which you likely want to set #attributes and #weight.
    // Each child element on the second level represents a table column
    // cell in the respective table row, which are render elements on
    // their own. For singleoutput elements, use the table cell itself
    // for the render element. If a cell should contain multiple elements,
    // simply use nested sub-keys to build the render element structure
    // for drupal_render() as you would everywhere else.
    $this->items = [];

    if (!empty($siteIdFrom) && !empty($siteIdTo)) {
      $this->items = $this->client->getAllBackupsOrphan($siteIdFrom, $siteIdTo);
    }

    // $rowPiece = $this->pagerArray($this->items, 10);
    foreach ($this->items as $id => $entity) {
      // Some table columns containing raw markup.
      $form['entities-table'][$id]['ID'] = [
        '#plain_text' => $entity->id,
      ];
      $form['entities-table'][$id]['nid'] = [
        '#plain_text' => $entity->nid,
      ];
      $form['entities-table'][$id]['uid'] = [
        '#plain_text' => $entity->uid,
      ];

      if (!$this->isObsolete($backupRetention, $entity->timestamp)) {
        $timestamp = date("Y/m/d H:i", $entity->timestamp);
      }
      else {
        $timestamp = '<span class="acsf-backup-manager-color-error">' . date("Y/m/d H:i", $entity->timestamp) . '</span>';
      }
      $form['entities-table'][$id]['timestamp'] = [
        '#markup' => $timestamp,
      ];
      $form['entities-table'][$id]['obsolete'] = [
        '#markup' => json_encode($this->isObsolete($backupRetention, $entity->timestamp)),
      ];
      $form['entities-table'][$id]['directory'] = [
        '#plain_text' => $entity->directory,
      ];
      $form['entities-table'][$id]['file'] = [
        '#plain_text' => $entity->file,
      ];
      if ($this->config->get('filesize')) {
        $tmpFileUrl = $this->client->backupGetTempLink($entity->nid, $entity->id);
        $fileSize = $this->client->backupGetFileSize($tmpFileUrl);

        $form['entities-table'][$id]['size'] = [
          '#plain_text' => !empty($fileSize['formatted']) ? $fileSize['formatted'] : '-',
        ];
      }
      $form['entities-table'][$id]['label'] = [
        '#plain_text' => $entity->label,
      ];
      $form['entities-table'][$id]['componentList'] = [
        '#markup' => implode("<br>", $entity->componentList),
      ];

      // Operations (dropbutton) column.
      $form['entities-table'][$id]['operations'] = [
        '#type' => 'operations',
        '#links' => [],
      ];
      $form['entities-table'][$id]['operations']['#links']['backup_download'] = [
        'title' => $this->t('Download'),
        'url' => Url::fromRoute(
          'acsf_backup_manager.action_controller_backupDownload',
          [
            'siteId' => $entity->nid,
            'backupId' => $entity->id,
          ],
          ['attributes' => ['target' => '_blank']]
        ),
      ];
      $form['entities-table'][$id]['operations']['#links']['backup_delete'] = [
        'title' => $this->t('Delete'),
        'url' => Url::fromRoute(
          'acsf_backup_manager.action_controller_backupDelete',
          [
            'siteId' => $entity->nid,
            'backupId' => $entity->id,
          ]),
      ];
    }

    $form['actions'] = ['#type' => 'actions'];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Apply to selected items'),
      // TableSelect: Enable the built-in form validation for #tableselect for
      // this form button, so as to ensure that the bulk operations form cannot
      // be submitted without any selected items.
      '#tableselect' => TRUE,
    ];

    // $form['pager'] = [
    // '#type' => 'pager',
    // ];
    return $form;
  }

  /**
   * Returns pager array.
   */
  public function pagerArray($items, $itemsPerPage) {
    // Get total items count.
    $total = count($items);
    // Get the number of the current page.
    $currentPage = $this->pagerManager->createPager($total, $itemsPerPage)
      ->getCurrentPage();
    // Split an array into chunks.
    $chunks = array_chunk($items, $itemsPerPage);
    // Return current group item.
    $currentPageItems = $chunks[$currentPage];
    return $currentPageItems;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $formValues = $form_state->getValues();
    $result = [];
    $items = $this->items;
    foreach ($formValues['entities-table'] as $value) {
      if (isset($value)) {
        $result[] = $items[$value];
      }
    }

    $submit = $form_state->getTriggeringElement()['#name'];

    if (count($result) == 0 && $submit !== 'filter') {
      \Drupal::messenger()
        ->addMessage($this->t('You should select at least one item.'), 'error');
    }
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $formValues = $form_state->getValues();
    $result = [];
    $items = $this->items;
    foreach ($formValues['entities-table'] as $value) {
      if (is_string($value)) {
        $result[] = $items[$value];
      }
    }

    $submit = $form_state->getTriggeringElement()['#name'];

    if ($submit == 'filter') {
      if ($formValues['site_id_to'] < $formValues['site_id_from']) {
        \Drupal::messenger()
          ->addMessage($this->t('Wrong ID diapason. "From site ID" should be less than "To site ID"'), 'error');
      }
      elseif (($formValues['site_id_to'] - $formValues['site_id_from']) > 500) {
        \Drupal::messenger()
          ->addMessage($this->t('Wrong ID diapason. The difference should be less than 500'), 'error');
      }
      else {
        $form_state->setRedirectUrl(Url::fromRoute(
          'acsf_backup_manager.admin_backups_orphan_form',
          [
            'siteIdFrom' => $formValues['site_id_from'],
            'siteIdTo' => $formValues['site_id_to'],
          ]
        ));
      }

    }

    if (!empty($formValues['batch_action']) && $submit !== 'filter') {
      // @todo Test all actions across the module
      switch ($formValues['batch_action']) {
        case 'delete_backups_backups':
          foreach ($result as $item) {
            $this->client->backupDelete($item->uid, $item->id);
          }

          break;

        case 'export_backups_backups':
          $content = "Site ID,Backup ID,User ID,Site,Domain,DB Name,Created,Obsolete,Directory,File";
          if ($this->config->get('filesize')) {
            $content .= ",Size";
          }
          $content .= ",Label,Component List\n";
          $backupRetention = $this->config->get('backup_retention');
          foreach ($result as $item) {
            $site = $this->client->getSite($item->nid);
            $domains = $site->domains;
            $created = date("Y/m/d H:i", $item->timestamp);
            $obsolete = json_encode($this->isObsolete($backupRetention, $item->timestamp));
            $label = str_replace(",", " ", $item->label);
            $componentList = implode("|", $item->componentList);
            $content .= "$item->nid,$item->id,$item->uid,$site->site,$domains[0],$site->db_name,$created,$obsolete,$item->directory,$item->file";
            if ($this->config->get('filesize')) {
              $tmpFileUrl = $this->client->backupGetTempLink($item->nid, $item->id);
              $fileSize = $this->client->backupGetFileSize($tmpFileUrl);

              $content .= "," . (!empty($fileSize['formatted']) ? $fileSize['formatted'] : '-');
            }
            $content .= ",$label,$componentList\n";
          }
          $responseData = [
            'filename' => '_backups.csv',
            'content' => $content,
          ];

          break;
      }

      if (isset($responseData)) {
        $apiUser = $this->config->get('api_user');
        $apiUrl = $this->config->get('api_url');
        $url = parse_url($apiUrl);
        $filename = $url['host'] . '_' . $apiUser . $responseData['filename'];

        $response = new Response($responseData['content']);
        $disposition = $response->headers->makeDisposition(ResponseHeaderBag::DISPOSITION_ATTACHMENT, $filename);
        $response->headers->set('Content-type', 'application/csv');
        $response->headers->set('Content-Disposition', $disposition);
        $response->headers->set('Content-Transfer-Encoding', 'binary');
        $response->headers->set('Content-length', strlen($responseData['content']));
        $form_state->setResponse($response);
      }
    }
    elseif ($submit !== 'filter') {
      \Drupal::messenger()
        ->addMessage($this->t('You should select certain action.'), 'error');
    }
  }

}
