<?php

namespace Drupal\acsf_backup_manager\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\acsf_backup_manager\Controller\ApiInterface;

/**
 * Class AdminSettingsForm.
 *
 * Provides the form for the module settings.
 */
class AdminSettingsForm extends ConfigFormBase {

  /**
   * The API client.
   *
   * @var \Drupal\acsf_backup_manager\Controller\ApiInterface
   */
  protected $client;

  /**
   * Constructs \Drupal\acsf_backup_manager\Form\AdminSettingsForm object.
   *
   * @param \Drupal\acsf_backup_manager\Controller\ApiInterface $client
   *   The API client.
   */
  public function __construct(ApiInterface $client) {
    $this->client = $client;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('acsf_backup_manager.api_client')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'acsf_backup_manager_admin_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'acsf_backup_manager.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config($this->getEditableConfigNames()[0]);

    $form['api_data'] = [
      '#type' => 'details',
      '#title' => $this->t('API settings'),
      '#description' => $this->t('You should have ACSF access to the API.'),
      '#open' => TRUE,
    ];

    $form['api_data']['api_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API base URL'),
      '#description' => $this->t('This is your ACSF environment url followed by /api/v#/. Refer to ACSF documentation to determine what api version number to call.'),
      '#default_value' => $config->get('api_url'),
      '#required' => TRUE,
    ];

    $form['api_data']['api_user'] = [
      '#type' => 'textfield',
      '#title' => $this->t('ACSF administrative user name'),
      '#description' => $this->t('This is used as your api login; only ACSF users with an administrative role can perform obtain an API key to run api calls, including those used for this module.'),
      '#default_value' => $config->get('api_user'),
      '#required' => TRUE,
    ];

    $form['api_data']['api_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API key'),
      '#default_value' => $config->get('api_key'),
      '#required' => TRUE,
    ];

    $form['api_data']['backup_scope'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Backup creating scope'),
      '#description' => $this->t('Determine what data you want to include in the backup archive when you create it.'),
      '#options' => [
        'database' => $this->t('Database'),
        'public files' => $this->t('Public files'),
        'private files' => $this->t('Private files'),
        'themes' => $this->t('Themes'),
      ],
      '#default_value' => $config->get('backup_scope'),
    ];

    $form['api_data']['limit'] = [
      '#type' => 'number',
      '#title' => $this->t('Limit of requesting items'),
      '#description' => $this->t('The amount of data that will be retrieved by one request (maximum 100 items). Does not affect the amount of data displayed in tables, but it does affect performance and the number of requests to the server.'),
      '#default_value' => $config->get('limit'),
      '#required' => TRUE,
    ];

    $form['api_data']['backup_retention'] = [
      '#type' => 'number',
      '#title' => $this->t('The number of days after which the backup is considered obsolete'),
      '#default_value' => $config->get('backup_retention'),
      '#required' => TRUE,
    ];

    $form['other'] = [
      '#type' => 'details',
      '#title' => $this->t('Other settings'),
      '#open' => TRUE,
    ];

    $form['other']['debug'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Debug'),
      '#description' => $this->t('Log messages will be added to the Drupal log'),
      '#default_value' => $config->get('debug'),
    ];
    $form['other']['filesize'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show backup archive count and filesize in the tables'),
      '#description' => $this->t('Please be careful using this option! It can dramatically slow down getting data procedure. You can judge file size by the "Components List" column.'),
      '#default_value' => $config->get('filesize'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    foreach ($form_state->getValues() as $key => $value) {
      switch ($key) {
        case 'limit':
          if ($value < 1) {
            $form_state->setErrorByName('limit', "Limit must be greater than zero.");
          }

          if ($value > 100) {
            \Drupal::messenger()
              ->addMessage("You should be careful with too big amount of data! It would be better to set limit less than 100.", 'warning');
          }
          break;
      }
    }
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config($this->getEditableConfigNames()[0])
      ->set('api_url', $form_state->getValue('api_url'))
      ->set('api_user', $form_state->getValue('api_user'))
      ->set('api_key', $form_state->getValue('api_key'))
      ->set('backup_scope', $form_state->getValue('backup_scope'))
      ->set('limit', $form_state->getValue('limit'))
      ->set('backup_retention', $form_state->getValue('backup_retention'))
      ->set('debug', $form_state->getValue('debug'))
      ->set('filesize', $form_state->getValue('filesize'))
      ->save();

    if ($this->client->checkConnection()) {
      \Drupal::messenger()
        ->addMessage($this->t('Connection test is passed.'), 'status');
    }
    else {
      \Drupal::messenger()
        ->addMessage($this->t('Connection test is failed! Check configuration is correct. See logs for error details.'), 'error');
    }
  }

}
