<?php

namespace Drupal\acsf_backup_manager\Form;

use Drupal\acsf_backup_manager\Controller\ApiInterface;
use Drupal\acsf_backup_manager\Controller\HelperTrait;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Link;
use Drupal\Core\Pager\PagerManagerInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\ResponseHeaderBag;
use Drupal\Core\Config\ConfigFactoryInterface;

/**
 * Class AdminSitesForm.
 *
 * Provides the form for the sites.
 */
class AdminSitesForm extends FormBase {
  use HelperTrait;

  /**
   * The API client.
   *
   * @var \Drupal\acsf_backup_manager\Controller\ApiInterface
   */
  protected $client;

  /**
   * The pager manager.
   *
   * @var \Drupal\Core\Pager\PagerManagerInterface
   */
  protected $pagerManager;

  /**
   * The items storage.
   *
   * @var array
   */
  protected $items;

  /**
   * File system service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * {@inheritdoc}
   */
  protected $config;

  /**
   * Constructs \Drupal\acsf_backup_manager\Form\AdminSettingsForm object.
   *
   * @param \Drupal\acsf_backup_manager\Controller\ApiInterface $client
   *   The API client.
   * @param \Drupal\Core\Pager\PagerManagerInterface $pagerManager
   *   The pager manager.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   *   Request stack variable.
   */
  public function __construct(ApiInterface $client, PagerManagerInterface $pagerManager, ConfigFactoryInterface $config_factory, RequestStack $requestStack) {
    $this->client = $client;
    $this->pagerManager = $pagerManager;
    $this->configFactory = $config_factory;
    $this->config = $this->configFactory->get('acsf_backup_manager.settings');
    $this->request = $requestStack->getCurrentRequest();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('acsf_backup_manager.api_client'),
      $container->get('pager.manager'),
      $container->get('config.factory'),
      $container->get('request_stack')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'acsf_backup_manager_admin_sites_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'acsf_backup_manager.sites',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $siteId = $this->request->get('siteId');

    $form['batch_action'] = [
      '#title' => $this->t("Action"),
      '#type' => 'select',
      '#options' => [
        '' => $this->t("-- Select action --"),
        'create_backup' => $this->t("Create backup"),
        'delete_backups_sites' => $this->t("Delete obsolete backups"),
        'export_sites_sites' => $this->t("Export sites report"),
        'export_backups_sites' => $this->t("Export backups report"),
      ],
    ];

    $form['entities-table'] = [
      '#type' => 'table',
      '#header' => [
        $this->t('Site ID'),
        $this->t('Name'),
        $this->t('Domain'),
        $this->t('DB name'),
        $this->t('Backups'),
        $this->t('Operations'),
      ],
      '#empty' => $this->t('There are no items.'),
      // TableSelect: Injects a first column containing the selection widget
      // into each table row.
      // Note that you also need to set #tableselect on each form submit
      // button that relies on non-empty selection values (see below).
      '#tableselect' => TRUE,
    ];
    $form['entities-table']['#header'] = [
      $this->t('Site ID'),
      $this->t('Name'),
      $this->t('Domain'),
      $this->t('DB name'),
    ];
    if ($this->config->get('filesize')) {
      $form['entities-table']['#header'][] = $this->t('Backups');
    }
    $form['entities-table']['#header'][] = $this->t('Operations');
    // Build the table rows and columns.
    // The first nested level in the render array forms the table row,
    // on which you likely want to set #attributes and #weight.
    // Each child element on the second level represents a table column
    // cell in the respective table row, which are render elements on
    // their own. For singleoutput elements, use the table cell itself
    // for the render element. If a cell should contain multiple elements,
    // simply use nested sub-keys to build the render element structure
    // for drupal_render() as you would everywhere else.
    if (!empty($siteId)) {
      $site = $this->client->getSite($siteId);
      $siteDomain = $site->domains;
      $site->domain = $siteDomain[0];
      $this->items[] = $site;
    }
    else {
      $this->items = $this->client->getAllSites();
    }

    // $rowPiece = $this->pagerArray($this->items, 10);
    foreach ($this->items as $id => $entity) {
      $form['entities-table'][$id]['site_id'] = [
        '#plain_text' => $entity->id,
      ];
      $form['entities-table'][$id]['site'] = [
        '#plain_text' => $entity->site,
      ];
      $form['entities-table'][$id]['domain'] = [
        '#markup' => Link::fromTextAndUrl(
          $entity->domain,
          Url::fromUri('https://' . $entity->domain, [
            'attributes' => [
              'target' => '_blank',
            ],
          ])
        )
          ->toString(),
      ];
      $form['entities-table'][$id]['db_name'] = [
        '#plain_text' => $entity->db_name,
      ];

      if ($this->config->get('filesize')) {
        $backups = $this->client->getSiteBackups($entity->id);
        $form['entities-table'][$id]['archives'] = [
          '#markup' => $backups ? count($backups) : '-',
        ];
      }
      // Operations (dropbutton) column.
      $form['entities-table'][$id]['operations'] = [
        '#type' => 'operations',
        '#links' => [],
      ];
      $form['entities-table'][$id]['operations']['#links']['backups_show'] = [
        'title' => $this->t('Show backups'),
        'url' => Url::fromRoute('acsf_backup_manager.admin_backups_form', ['siteId' => $entity->id]),
      ];
      $form['entities-table'][$id]['operations']['#links']['backup_create'] = [
        'title' => $this->t('Create backup'),
        'url' => Url::fromRoute('acsf_backup_manager.action_controller_backupCreate', ['siteId' => $entity->id]),
      ];
      $form['entities-table'][$id]['operations']['#links']['backups_delete'] = [
        'title' => $this->t('Delete obsolete backups'),
        'url' => Url::fromRoute('acsf_backup_manager.action_controller_backupsDelete', ['siteId' => $entity->id]),
      ];
    }
    $form['actions'] = ['#type' => 'actions'];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Apply to selected items'),
      // TableSelect: Enable the built-in form validation for #tableselect for
      // this form button, so as to ensure that the bulk operations form cannot
      // be submitted without any selected items.
      '#tableselect' => TRUE,
    ];

    // $form['pager'] = [
    // '#type' => 'pager',
    // ];
    return $form;
  }

  /**
   * Returns pager array.
   */
  public function pagerArray($items, $itemsPerPage) {
    // Get total items count.
    $total = count($items);
    // Get the number of the current page.
    $currentPage = $this->pagerManager->createPager($total, $itemsPerPage)
      ->getCurrentPage();
    // Split an array into chunks.
    $chunks = array_chunk($items, $itemsPerPage);
    // Return current group item.
    $currentPageItems = $chunks[$currentPage];
    return $currentPageItems;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $formValues = $form_state->getValues();
    $result = [];
    $items = $this->items;
    foreach ($formValues['entities-table'] as $value) {
      if (!empty($value)) {
        $result[] = $items[$value];
      }
    }

    if (count($result) == 0) {
      \Drupal::messenger()
        ->addMessage($this->t('You should select at least one item.'), 'error');
    }
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $formValues = $form_state->getValues();
    $result = [];
    $items = $this->items;
    foreach ($formValues['entities-table'] as $value) {
      if (is_string($value)) {
        $result[] = $items[$value];
      }
    }

    if (!empty($formValues['batch_action'])) {
      switch ($formValues['batch_action']) {
        case 'create_backup':
          foreach ($result as $item) {
            $this->client->backupCreate($item->id);
          }

          break;

        case 'delete_backups_sites':
          foreach ($result as $item) {
            $this->client->backupsDelete($item->id);
          }

          break;

        case 'export_sites_sites':
          $content = "Site ID,Site,Domain,DB Name,Backups\n";
          foreach ($result as $item) {
            $backups = count($this->client->getSiteBackups($item->id));
            $content .= "$item->id,$item->site,$item->domain,$item->db_name,$backups\n";
          }
          $responseData = [
            'filename' => '_sites.csv',
            'content' => $content,
          ];

          break;

        case 'export_backups_sites':
          $content = "Site ID,Backup ID,User ID,Site,Domain,DB Name,Created,Obsolete,Directory,File";
          if ($this->config->get('filesize')) {
            $content .= ",Size";
          }
          $content .= ",Label,Component List\n";
          $backupRetention = $this->config->get('backup_retention');
          foreach ($result as $item) {
            $backups = $this->client->getSiteBackups($item->id);
            foreach ($backups as $backup) {
              $created = date("Y/m/d H:i", $backup->timestamp);
              $obsolete = json_encode($this->isObsolete($backupRetention, $backup->timestamp));
              $label = str_replace(",", " ", $backup->label);
              $componentList = implode("|", $backup->componentList);
              $content .= "$backup->nid,$backup->id,$backup->uid,$item->site,$item->domain,$item->db_name,$created,$obsolete,$backup->directory,$backup->file";
              if ($this->config->get('filesize')) {
                $tmpFileUrl = $this->client->backupGetTempLink($backup->nid, $backup->id);
                $fileSize = $this->client->backupGetFileSize($tmpFileUrl);

                $content .= "," . (!empty($fileSize['formatted']) ? $fileSize['formatted'] : '-');
              }
              $content .= ",$label,$componentList\n";
            }
          }
          $responseData = [
            'filename' => '_sites_backups.csv',
            'content' => $content,
          ];

          break;
      }

      if (isset($responseData)) {
        $apiUser = $this->config->get('api_user');
        $apiUrl = $this->config->get('api_url');
        $url = parse_url($apiUrl);
        $filename = $url['host'] . '_' . $apiUser . $responseData['filename'];

        $response = new Response($responseData['content']);
        $disposition = $response->headers->makeDisposition(ResponseHeaderBag::DISPOSITION_ATTACHMENT, $filename);
        $response->headers->set('Content-type', 'application/csv');
        $response->headers->set('Content-Disposition', $disposition);
        $response->headers->set('Content-Transfer-Encoding', 'binary');
        $response->headers->set('Content-length', strlen($responseData['content']));
        $form_state->setResponse($response);
      }
    }
    else {
      \Drupal::messenger()
        ->addMessage($this->t('You should select certain action.'), 'error');
    }
  }

}
