<?php

namespace Drupal\acsf_backup_manager\Form;

use Drupal\acsf_backup_manager\Controller\ApiInterface;
use Drupal\acsf_backup_manager\Controller\HelperTrait;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Pager\PagerManagerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Class AdminStatisticForm.
 *
 * Provides the form for the backups.
 */
class AdminStatisticForm extends FormBase {

  use HelperTrait;

  /**
   * The API client.
   *
   * @var \Drupal\acsf_backup_manager\Controller\ApiInterface
   */
  protected $client;

  /**
   * The pager manager.
   *
   * @var \Drupal\Core\Pager\PagerManagerInterface
   */
  protected $pagerManager;

  /**
   * The request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * The items storage.
   *
   * @var array
   */
  protected $items;

  /**
   * File system service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * {@inheritdoc}
   */
  protected $config;

  /**
   * CallServicesController constructor.
   *
   * @param \Drupal\acsf_backup_manager\Controller\ApiInterface $client
   *   The API client.
   * @param \Drupal\Core\Pager\PagerManagerInterface $pagerManager
   *   The pager manager.
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   *   Request stack variable.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   */
  public function __construct(ApiInterface $client, PagerManagerInterface $pagerManager, RequestStack $requestStack, ConfigFactoryInterface $config_factory) {
    $this->client = $client;
    $this->pagerManager = $pagerManager;
    $this->request = $requestStack->getCurrentRequest();
    $this->configFactory = $config_factory;
    $this->config = $this->configFactory->get('acsf_backup_manager.settings');
  }

  /**
   * Services DI.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   Container DI.
   *
   * @return static
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('acsf_backup_manager.api_client'),
      $container->get('pager.manager'),
      $container->get('request_stack'),
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'acsf_backup_manager_admin_statistic_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'acsf_backup_manager.statistic',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $backupRetention = $this->config->get('backup_retention');
    $form['legend'] = [
      '#markup' => '<span class="acsf-backup-manager-color-error">' . $this->t('Marked') . '</span> - ' . $this->t('are obsolete archives. You can define in the settings which archives are considered obsolete. Currently it is @backupRetention days.', ['@backupRetention' => $backupRetention]),
    ];

    $form['entities-table'] = [
      '#type' => 'table',
      '#empty' => $this->t('There are no items.'),
      // TableSelect: Injects a first column containing the selection widget
      // into each table row.
      // Note that you also need to set #tableselect on each form submit
      // button that relies on non-empty selection values (see below).
      '#tableselect' => FALSE,
    ];

    $showFileSize = $this->config->get('filesize');
    $form['entities-table']['#header'] = [
      $this->t('Site groups'),
      $this->t('Site collections'),
      $this->t('Sites'),
      $this->t('Backups obsolete'),
      $this->t('Backups fresh'),
      $this->t('Backups total'),
    ];

    // Build the table rows and columns.
    // The first nested level in the render array forms the table row,
    // on which you likely want to set #attributes and #weight.
    // Each child element on the second level represents a table column
    // cell in the respective table row, which are render elements on
    // their own. For singleoutput elements, use the table cell itself
    // for the render element. If a cell should contain multiple elements,
    // simply use nested sub-keys to build the render element structure
    // for drupal_render() as you would everywhere else.
    $this->items = [];

    // $this->items = $this->client->getAllBackups();
    $groups = $this->client->getAllSiteGroups();
    $collections = $this->client->getAllSiteCollections();
    $sites = $this->client->getAllSites();
    $backups = $this->client->getAllBackups();
    $backupsInfo = [
      'fresh' => [
        'count' => 0,
        'size' => 0,
        'size-formatted' => 0,
      ],
      'total' => [
        'count' => 0,
        'size' => 0,
        'size-formatted' => 0,
      ],
    ];

    foreach ($backups as $backup) {
      $backupsInfo['total']['count']++;

      $backupSize = FALSE;
      if ($showFileSize) {
        $tmpLink = $this->client->backupGetTempLink($backup->nid, $backup->id);
        if ($tmpLink) {
          $backupSize = $this->client->backupGetFileSize($tmpLink);
          $backupsInfo['total']['size'] += $backupSize['original'];
        }
      }

      if (!$this->isObsolete($backupRetention, $backup->timestamp)) {
        $backupsInfo['fresh']['count']++;
        if ($backupSize) {
          $backupsInfo['fresh']['size'] += $backupSize['original'];
        }
      }
    }
    $backupsInfo['fresh']['size-formatted'] = $this->getFormattedFileSize($backupsInfo['fresh']['size']);
    $backupsInfo['total']['size-formatted'] = $this->getFormattedFileSize($backupsInfo['total']['size']);

    $backupsInfo['obsolete']['count'] = $backupsInfo['total']['count'] - $backupsInfo['fresh']['count'];
    $backupsInfo['obsolete']['size'] = $backupsInfo['total']['size'] - $backupsInfo['fresh']['size'];
    $backupsInfo['obsolete']['size-formatted'] = $this->getFormattedFileSize($backupsInfo['obsolete']['size']);

    $form['entities-table'][0]['groups'] = [
      '#plain_text' => !empty($groups) ? count($groups) : '-',
    ];
    $form['entities-table'][0]['collections'] = [
      '#plain_text' => !empty($collections) ? count($collections) : '-',
    ];
    $form['entities-table'][0]['sites'] = [
      '#plain_text' => !empty($sites) ? count($sites) : '-',
    ];
    $form['entities-table'][0]['backups_obsolete'] = [
      '#markup' => '<span class="acsf-backup-manager-color-error">' . $backupsInfo['obsolete']['count'] . ($showFileSize ? ' / ' . $backupsInfo['obsolete']['size-formatted'] : '') . '</span>',
    ];
    $form['entities-table'][0]['backups_fresh'] = [
      '#markup' => $backupsInfo['fresh']['count'] . ($showFileSize ? ' / ' . $backupsInfo['fresh']['size-formatted'] : ''),
    ];
    $form['entities-table'][0]['backups_total'] = [
      '#markup' => $backupsInfo['total']['count'] . ($showFileSize ? ' / ' . $backupsInfo['total']['size-formatted'] : ''),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    return TRUE;
  }

}
